// The following ifdef block is the standard way of creating macros which make exporting
// from a DLL simpler. All files within this DLL are compiled with the INTERFACE_EXPORTS
// symbol defined on the command line. This symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see
// INTERFACE_API functions as being imported from a DLL, whereas this DLL sees symbols
// defined with this macro as being exported.
#ifdef INTERFACE_EXPORTS
#define INTERFACE_API
#else
#define INTERFACE_API
#endif


#include "myusb.h"
#include "xspec.h"
#define INFINITE 0
#define ONE_SEC 1000
#define TWO_SEC 2000
#define FIVE_SEC 5000


// Scan for XPSEC devices and create a internal linked list of attached devices.
// Call this first before any other function.
// return number of XSPEC devices ; <0 error.
extern "C" INTERFACE_API int XSPEC_scan_usb_for_spec(void);
//print all XSPEC USB structure
extern "C" INTERFACE_API void XSPEC_print_Spec();
// call to delete allocated data structures when program is shutting down.
extern "C" INTERFACE_API void XSPEC_usb_cleanup();
//return number of XPSECs connected. call scan_sub_for_spec first.
extern "C" INTERFACE_API int XSPEC_getNumConnectedSpectrometers(void);
//open the XSPEC device by devnum, devnum is stored in the USB_SPEC structure
extern "C" INTERFACE_API usb_dev_handle *XSPEC_open_Spec(char devnum);
//call this to close the usb device. 0==success, < 0 on error.
extern "C" INTERFACE_API int XSPEC_close_Spec(USB_SPEC *node);

// retrieve the XSPEC device data structure by index.
// 0==success, -1 error
extern "C" INTERFACE_API int XSPEC_get_Spec(unsigned int index, USB_SPEC *node);

// set the integration time in usec for the current_devnum
// return the number of bytes sent for the command, must be larger than 0.
extern "C" INTERFACE_API int XSPEC_setIntegrationTime(USB_SPEC *ptr,unsigned int time_us);

// return the integration time in usec for the current_devnum
// -1==failure,
extern "C" INTERFACE_API int XSPEC_getIntegrationTime(USB_SPEC *ptr);

// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// pre-allocate the space for p_data, it is 128bytes. cast to the structure defined in myusb.h
// -1==failure, 0==success
extern "C" INTERFACE_API int XSPEC_get_SpecConfig0123(USB_SPEC *ptr,char *p_data, int index);

// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// create the data for p_data, it is 128bytes. use get_SpecConfig() to get the entire structure. Modify the one you need
// to change and put the data structure back using this function.
// return number of bytes sent, -1==failure,
extern "C" INTERFACE_API int XSPEC_write_SpecConfig(USB_SPEC *ptr,char *p_data, int index);
//only update the setting0 in memory, not update the flash
extern "C" INTERFACE_API int XSPEC_update_config_mem(USB_SPEC *ptr,char *p_data);
// download program page by page into the flash. index start from 0, page_size is the flash page size, it may differ depending on which ucontroller is used.  128 or 256
// return number of bytes sent, -1==failure,
extern "C" INTERFACE_API int XSPEC_write_SpecProg(USB_SPEC *ptr,char *p_data, unsigned int page_size, int index);
//only send the read command,
extern "C" INTERFACE_API int XSPEC_get_SpecConfig_async(USB_SPEC *ptr);


//retrieve the 4 data structures stored in the flash and save them locally.
// 0==success, -1==failure.
extern "C" INTERFACE_API int XSPEC_get_SpecConfig(USB_SPEC *ptr);
//retrieve a page from the flash
// 0==success, -1==failure.
extern "C" INTERFACE_API int XSPEC_get_SpecProg(USB_SPEC *ptr, unsigned short page);

// dump the  4 data structures using printf
// // 0==success, else==failure
extern "C" INTERFACE_API int XSPEC_print_config(USB_SPEC *ptr);


// pre-allocate BUF_SIZE for the data_array, id is an unique ID to distingusish multiple users accessing the same device.
// passed integ_time help determine how long to wait for the data. in usec; integ_time/1000+1000ms
// return number of bytes received. <=0 indicates error.
/*
 Each pixel is 2 bytes, (16bits).
PIXEL 19-31 (including 31) are dark pixels.   Normally , average is taken and used for subtraction.
Start pixel is 35,
  for(tmp=START_DARK;tmp<START_DARK+TOTAL_DARK;tmp++)  //dark from 19 to 31
{
   ptr1->ground+=buffer[tmp];

  }
ptr1->ground=ptr1->ground/TOTAL_DARK;


For data:

for(tmp=START_PIXEL;tmp<START_PIXEL+VALID_PIXELS;tmp++)

//defined in spec.h

#define START_PIXEL 36//sensor datasheet says 32, might caused by read delay.
#define VALID_PIXELS 3648  //3683
#define LAST_PIXEL 3682
#define TOTAL_PIXELS 3694
#define START_DARK 19
#define TOTAL_DARK 13

Returned spectrum format:
2 byte each. (short)

[0]PACKET_START, (0xE1)
[1]TYPE(TRIG_CMD 0x51),
[2]SIZE(received size in bytes),  3684*2 for TCD1304DG
[3]Type of detector. TCD1304=0, Hama S1115X 1
[START_DARK-1] Average Dark current,  avg_dark
[START_DARK-2] Number of spectrums where the Average Dark Current was taken.  avg_dark_counter
            dark_sum=0;
            for(dark_value=START_DARK; dark_value<START_DARK+TOTAL_DARK; dark_value++)
            {
                dark_sum+=ptr[dark_index][dark_value];
            }
            dark_sum1=dark_sum/TOTAL_DARK;
            avg_dark=(avg_dark*avg_dark_counter+dark_sum1)/(avg_dark_counter+1);
            if(avg_dark_counter<1023)
                avg_dark_counter++;

[START_DARK-3] number of spectrum average in the firmware.


4byte each, (Int)
[2]  Number of data sets were ready to be sent, due to the transmission speed, some might got dropped. Used in continuous read.
[3], Data sets sent through the USB. Used in continuous read.
[4], start time, Used in continuous read.  divide by 3 to us
[5], integration time, if integration < 3720, subtract 0x80000000, else divide by 3
		if(current_data->real_integ_time & 0x80000000)
			temp.Format(_T("S%dus"), current_data->real_integ_time& 0x7FFFFFFF);
		else
		{
			double dtemp;
			unsigned int integ;
			dtemp=(double)current_data->real_integ_time;
			dtemp=dtemp/3.0+0.5;
			integ=(unsigned int)dtemp;
			temp.Format(_T("N%dus"), integ);
		}

[6] ID, passed in XSPEC_measure. if EXTERNAL Trig, default is  #define EXT_TRIG_ID 0xE1234567, overwritten by ID in  XSPEC_measure
[7] TEMPERATURE
            double temperature; //0.3v at 25c, 5.7X amplification 3.3=1023
            unsigned int bits;

            bits=ptr_data[7];
            temperature=bits& 0xFFFF;
            bits=bits>>16;
            if(bits==0)
            {
                temperature=temperature/1023.0*3.3;
                temperature=((temperature/5.7)-0.3)*1000.0+25.0;
            }
            else
            {
                double max;
                double volt;
                max=pow(2.0,(int)bits&0xff);
                volt=bits>>8;
                volt=volt/10;
                temperature=(double)temperature/max*volt;
                temperature=((temperature/5.7)-0.3)*1000.0+25.0;
            }

            TRACE(_T("***********************************temp =%x %lf\n"),ptr_data[7],temperature );

*/

extern "C" INTERFACE_API int XSPEC_measure(USB_SPEC *ptr,unsigned short *data_array, unsigned int id, unsigned int integ_time);
//only send the command, return right away, return bytes sent.
extern "C" INTERFACE_API int XSPEC_measure_async(USB_SPEC *ptr,unsigned int id);
// initiate continous read mode. Only send a command. returns bytes sent
// User needs to implement their own method (thread) to retrieve and process continous data flow.
extern "C" INTERFACE_API int XSPEC_continous_measure_async(USB_SPEC *ptr,unsigned int id);
//stop spectrometer from sending data.
extern "C" INTERFACE_API int XSPEC_measure_stop(USB_SPEC *ptr);
extern "C" INTERFACE_API int XSPEC_pwm_start(USB_SPEC *ptr, int which);
extern "C" INTERFACE_API int XSPEC_pwm_stop(USB_SPEC *ptr, int which);
// Used to help user retrieve continous data in a thread.
// Put this function in a while loop inside a thread.
// return number of bytes read,  <=0 indicate timeout.
extern "C" INTERFACE_API int XSPEC_continous_wait(USB_SPEC *ptr,unsigned short *data_array, unsigned int integration_time);
// Restart the device. goto address "0"
extern "C" INTERFACE_API int XSPEC_reset(USB_SPEC *node);
extern "C" INTERFACE_API int XSPEC_upgrade(USB_SPEC *node);
//only supported with DAC installed. adjust the DAC to reach average count of 90 for the dark signal
extern "C" INTERFACE_API int XSPEC_adjust_bias(USB_SPEC *node);
extern "C" INTERFACE_API  double XSPEC_get_temperature(unsigned int bits);
//set or unset flash lamp syn  , pass 1 or 0. 1 to enable, 0 to disable. 
extern "C" INTERFACE_API int XSPEC_flash_Sync(USB_SPEC *node, unsigned char val);




#if 0
//RS232
#define PIXELS 3684
#define TCD1304_DATA_START 0
#define TCD1304_DATA_END 3647
extern "C" INTERFACE_API int XSPEC_rs232_upgrade(HANDLE comport);
//open the XSPEC device by comm port number 0,1,2,3,4...
extern "C" INTERFACE_API HANDLE XSPEC_rs232_open(unsigned char devnum);
//call this to close the rs232. 0==success, < 0 on error.
extern "C" INTERFACE_API int XSPEC_rs232_close(HANDLE comport);



// set the integration time in usec for the current_devnum
// return the number of bytes sent for the command, must be larger than 0.
extern "C" INTERFACE_API int XSPEC_rs232_setIntegrationTime(HANDLE comport,unsigned int time_us);
// set the integration time in usec for the current_devnum, stays in FLASH
// return the number of bytes sent for the command, must be larger than 0.
extern "C" INTERFACE_API int XSPEC_rs232_setIntegrationTimeFlash(HANDLE comport,unsigned int time_us);
// set the integration mode for the current_devnum, stays in FLASH
// mode 0: normal. software
// mode 1: rising external edge
// mode 2: falling external edge
// return the number of bytes sent for the command, must be larger than 0.
extern "C" INTERFACE_API int XSPEC_rs232_setModeFlash(HANDLE comport,unsigned int mode);
// set the firmware averaging for the current_devnum, stays in FLASH
// return the number of bytes sent for the command, must be larger than 0.
extern "C" INTERFACE_API int XSPEC_rs232_setAverageFlash(HANDLE comport,unsigned int average);

// return the integration time in usec for the current_devnum
// -1==failure,
extern "C" INTERFACE_API int XSPEC_rs232_getIntegrationTime(HANDLE comport);

// return the firmware average number.
// -1==failure,
extern "C" INTERFACE_API int XSPEC_rs232_getFirmwareAverage(HANDLE comport);
// return the trigger mode
// -1==failure,
extern "C" INTERFACE_API int XSPEC_rs232_getMode(HANDLE comport);
// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// pre-allocate the space for p_data, it is 128bytes. cast to the structure defined in myusb.h
// -1==failure, 0==success
extern "C" INTERFACE_API int XSPEC_rs232_get_SpecConfig0123(HANDLE comport,char *p_data, int index);

// 4 data structure stored in the flash. return one of them, use index (0,1,2,3) to indicate which one.
// create the data for p_data, it is 128bytes. use get_SpecConfig() to get the entire structure. Modify the one you need
// to change and put the data structure back using this function.
// return number of bytes sent, -1==failure,
extern "C" INTERFACE_API int XSPEC_rs232_write_SpecConfig(HANDLE comport,char *p_data, int index);
//only update the setting0 in memory, not update the flash
extern "C" INTERFACE_API int XSPEC_rs232_update_config_mem(HANDLE comport,char *p_data);




//retrieve the 4 data structures stored in the flash and save them locally.
// 0==success, -1==failure.
extern "C" INTERFACE_API int XSPEC_rs232_get_SpecConfig(HANDLE comport);

// dump the  4 data structures using printf
// // 0==success, else==failure
extern "C" INTERFACE_API int XSPEC_rs232_print_config(HANDLE comport);


// pre-allocate BUF_SIZE for the data_array, id is an unique ID to distingusish multiple users accessing the same device.
// passed integ_time help determine how long to wait for the data. in usec; integ_time/1000+1000ms
// return number of bytes received. <=0 indicates error.
extern "C" INTERFACE_API int XSPEC_rs232_measure(HANDLE comport,unsigned short *data_array, unsigned int id, unsigned int integ_time);

// pre-allocate BUF_SIZE for the data_array, id is an unique ID to distingusish multiple users accessing the same device.
// passed integ_time help determine how long to wait for the data. in usec; integ_time/1000+1000ms
// pass rs_start and rs_end 36-3683 for normal operation. SAM3U
// return number of bytes received. <=0 indicates error.
extern "C" INTERFACE_API int XSPEC_rs232_measure_partial(HANDLE comport,unsigned short *data_array, unsigned int id, unsigned int integ_time, unsigned short rs_start, unsigned short rs_end);

extern "C" INTERFACE_API int XSPEC_rs232_pwm_start(HANDLE comport, int which);
extern "C" INTERFACE_API int XSPEC_rs232_pwm_stop(HANDLE comport, int which);
// stop transmission,
extern "C" INTERFACE_API int XSPEC_rs232_abort(HANDLE comport);
// Restart the device. goto address "0"
extern "C" INTERFACE_API int XSPEC_rs232_reset(HANDLE comport);

extern "C" INTERFACE_API int XSPEC_rs232_measure_stop(HANDLE comport);
extern "C" INTERFACE_API int XSPEC_rs232_get_SpecProg(HANDLE comport, unsigned short page);
extern "C" INTERFACE_API int XSPEC_rs232_upgrade(HANDLE comport);
//only supported with DAC installed. adjust the DAC to reach average count of 90 for the dark signal
extern "C" INTERFACE_API int XSPEC_rs232_adjust_bias(HANDLE comport);






extern "C" INTERFACE_API int XSPEC_eth_udp_setIntegrationTime(SOCKET connect_socket, unsigned int time_us, sockaddr_in *sock_addr, int size);
extern "C"  INTERFACE_API int XSPEC_eth_udp_getIntegrationTime(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C"  INTERFACE_API int XSPEC_eth_udp_print_config(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_get_SpecConfig0123(SOCKET connect_socket, char *p_data, int index, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_update_config_mem(SOCKET connect_socket,  char *p_data, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_write_SpecConfig(SOCKET connect_socket, char *p_data, int index, sockaddr_in *sock_addr, int size);



extern "C" INTERFACE_API int XSPEC_eth_udp_reset(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_pwm_start(SOCKET connect_socket, int which, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_pwm_stop(SOCKET connect_socket, int which, sockaddr_in *sock_addr, int size);

extern "C" INTERFACE_API int XSPEC_eth_udp_get_SpecConfig_async(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_get_SpecConfig(SOCKET connect_socket, sockaddr_in *sock_addr, int size);

extern "C" INTERFACE_API SOCKET XSPEC_eth_udp_open_Spec(char *ip, int port);

extern "C" INTERFACE_API SOCKET XSPEC_eth_udp_close_Spec(SOCKET connect_socket);

extern "C" INTERFACE_API int XSPEC_eth_udp_get_SpecProg(SOCKET connect_socket, unsigned short page, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_upgrade(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_write_SpecProg(SOCKET connect_socket,char *p_data, unsigned int page_size, int index, sockaddr_in *sock_addr, int size);

extern "C" INTERFACE_API int XSPEC_eth_udp_measure(SOCKET connect_socket,  unsigned short *data_array, unsigned int id, unsigned int integ_time, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_measure_stop(SOCKET connect_socket, struct sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_measure_async(SOCKET connect_socket, unsigned int id, struct sockaddr_in *sock_addr, int len);
extern "C" INTERFACE_API int XSPEC_eth_udp_continous_measure_async(SOCKET connect_socket, unsigned int id, struct sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_continous_wait(SOCKET connect_socket, unsigned short *data_array);
extern "C" INTERFACE_API int XSPEC_eth_udp_find_dev(SOCKET connect_socket, sockaddr_in *sock_addr, int size);
extern "C" INTERFACE_API int XSPEC_eth_udp_change_ip(SOCKET connect_socket, sockaddr_in *sock_addr, int size, unsigned char *old_ip, unsigned char *new_ip, unsigned int port);
extern "C" INTERFACE_API int XSPEC_eth_adjust_bias(SOCKET connect_socket, struct sockaddr_in *sock_addr, int size);
#endif
