/* the device's vendor and product id */
#ifndef __XSPEC_H__
#define __MYRS232_H__ 


#define FLASH_PAGE_SIZE 128  //in bytes
#define BUF_SIZE 16000
#define RING_SIZE (1024*128)


#define TRIG_MODE_MAX 2
#define TRIG_MODE_NORMAL 0
#define TRIG_MODE_EXT_HARD_RISING 1
#define TRIG_MODE_EXT_HARD_FALLING 2

#define TRIG_MODE_LIBS_MAX 3
#define TRIG_MODE_LIBS_NORMAL 0
#define TRIG_MODE_LIBS_SHUTTER 1
#define TRIG_MODE_EXT_LIBS_RISING 2
#define TRIG_MODE_EXT_LIBS_FALLING 3

#define ST_SKIP_MIN 0
#define ST_SKIP_MAX 6000000
#define READ_COUNT_MIN 0
#define READ_COUNT_MAX 1000000
#define EXT_TRIG_DELAY_MIN 0
#define EXT_TRIG_DELAY_MAX 65000000
#define INT_TIME_LIBS_NORMAL_MIN 3700
#define INT_TIME_LIBS_SHUTTER_MIN 3

#define INT_TIME_NORMAL_MIN 10
#define INT_S1115X_TIME_NORMAL_MIN 2
#define INT_TIME_NORMAL_MAX 65000000

#define INT_S10420_TIME_NORMAL_MIN 9400

#define RUNNING_AVG_MIN 1
#define RUNNING_AVG_MAX 2000

#define PACKET_START 0xE1
#define ACQ_STATUS_CMD       0x50
#define TRIG_CMD             0x51
#define SET_DARK_CMD         0x52
#define SEND_CMD             0x53
#define SET_ITIME_CMD        0x54
#define ABORT_CMD            0x55
#define READ_NVMEM_CMD       0x56
#define WRITE_NVMEM_CMD      0x57
#define WRITE_SETTING_CMD    0x58  //update memory only, not flash
#define WRITE_PROG_CMD    0x59



#define READ_TEMP          0x5a
#define SET_PULSEMODE_CMD    0x5b
#define READ_VERSION_CMD     0x5d
#define READ_TEMP_CMD        0x5e
#define USB_RESET  0x5F
#define TRIG_CMD_REPEAT            0x60
#define READ_NVMEM_PRG       0x61
#define SET_ITIME_NOFLASH_CMD        0x62
#define START_PWM_CMD        0x63
#define END_PWM_CMD        0x64
#define UPGRADE_CMD        0x65
#define ADJUST_BIAS_CMD        0x66
//command from RS232 , no repeat

#define RS_TRIG_CMD 0x80
#define RS_SET_ITIME_CMD 0x81
#define RS_READ_NVMEM_CMD 0x82
#define RS_WRITE_NVMEM_CMD 0x83
#define RS_START_PWM_CMD 0x84
#define RS_END_PWM_CMD 0x85
#define RS_ABORT_CMD 0x86
#define RS_USB_RESET 0x87
#define RS_WRITE_SETTING_CMD    0x88  //update memory only, not flash
#define RS_WRITE_PROG_CMD    0x89
#define RS_READ_NVMEM_PRG  0x8A
#define RS_ADJUST_BIAS_CMD  0x8B
#define RS_TRIG_CMD_PARTIAL 0x8C

#define ET_TRIG_CMD 0x90
#define ET_SET_ITIME_CMD 0x91
#define ET_READ_NVMEM_CMD 0x92
#define ET_WRITE_NVMEM_CMD 0x93
#define ET_START_PWM_CMD 0x94
#define ET_END_PWM_CMD 0x95
#define ET_ABORT_CMD 0x96
#define ET_USB_RESET 0x97
#define ET_TRIG_CMD_REPEAT            0x98
#define ET_READ_NVMEM_PRG       0x99
#define ET_WRITE_SETTING_CMD    0x9a  //update memory only, not flash
#define ET_WRITE_PROG_CMD    0x9b
#define ET_UPGRADE_CMD        0x9c
#define ET_FIND_DEV_CMD       0x9D
#define ET_CHANGE_IP_CMD       0x9E
#define ET_ADJUST_BIAS_CMD  0x9F
// acq. status
#define WG_ACQ_DONE          8           // acq. finished
#define WG_ACQ_ABORT         4           // acq. aborted
#define WG_ACQ_WAIT_HWTRIG   1           // waiting for hardware trigger
#define WG_ACQ_INTEGRATING   2           // sensor is integrating
#define WG_ACQ_WAIT_SFTRIG	 3




typedef struct {
  unsigned int integration_time; //for regular mode 4
  unsigned int int_mode; //mode , normal or e-shutter mode 12, used as 0-63 for sub-microsecond control
  unsigned int trig_mode; //trigger mode; 0-4 16
  unsigned int extern_trigger_delay; // delay time in 0.5us for external trigger 20
  unsigned int st_skip; //24
  unsigned int dark_offset; //28
  unsigned int LAMP_PWM0; // duty and period
  unsigned int LAMP_PWM1; // scale, div, lowhigh
  unsigned int  multiple_count;         //   0xFFFF means infinite 40
  unsigned int  lamp_mode;    //44
  unsigned int dir_port; //48
  unsigned int port; //52
  unsigned int temp;
  unsigned int flash_size;
  unsigned int flash_page_size;
  unsigned int flash_setting_start;
  unsigned int start_auto_adj_dc_bias;	
  unsigned int average;
	unsigned int int_time;
	unsigned int trigger_out;  
	/*
		 -- 31/30 mode 11=external trigger, 01 shutter mode, 00 normal mode
		 -- bit 29-28 out 1 mode
		 -- bit 27-26 out 2 mode
		 -- bit 25-24 out 3 mode
		 -- bit 23-21, default value for out1,2,3
		 -- bit 20, 0 rising ext edge, 1 falling ext edge
		 -- bit 15-0, skip number
		 */
	unsigned int shutter_idle_time;
	unsigned int out1_delay;
	unsigned int out1_width;	
	unsigned int out2_delay;
	unsigned int out2_width;
	unsigned int out3_delay;
	unsigned int out3_width;
    unsigned int reserve[5];
  
} SYSTEM_SETTING_TYPE;
typedef struct {
  unsigned int description[4];//16
  unsigned int firmware_version[4];//32
  unsigned int serial[4];//48
  unsigned int detector[4];//64
  unsigned int grating[4];//80
  unsigned int entrance_filter[4];//96
  unsigned int slit_size[4];//112
  unsigned int sorting_filter[3];//124
  unsigned int pixels; //128
} SYSTEM_SETTING_STRING;

#define NUMBER_COEF 6
#define NUMBER_COEF_MAX 12

typedef struct {
    double wave_start; // 8
    double wave_end;  // 16
    double user_coef[NUMBER_COEF];// 64
    double system_coef[NUMBER_COEF];// 112 
    unsigned int BIAS_PWM; //    116
    unsigned short coef_order;   //116
    unsigned short reserve;   //120    
    unsigned short dark_start;  //118
    unsigned short dark_end;     //120
    unsigned short data_start;    //122
    unsigned short data_end;     //124
} SYSTEM_SETTING_COEF;


typedef struct {
    unsigned int user_name[16];//16
    unsigned int date_string[8];//16
    unsigned int time_string[8];//16
} SYSTEM_SETTING_NAME;
typedef struct {
    unsigned char  mac[6];
	unsigned char  ip[4];
	unsigned char  gate[4];
	unsigned char  mask[4];
	unsigned int   tcp_port;
	unsigned int   http_port;
	unsigned char use_this;
	unsigned char reserved[101];
} SYSTEM_SETTING_TCP;
typedef struct 
{
    double detector_coef[16];// 112 
} SYSTEM_LINEARITY_COEF;



#endif


