// xspec_control.cpp : Defines the entry point for the console application.
//

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <unistd.h>

#include "interface.h"
bool verbose;
void PRINT(const char *format, ...);
void printusage();
void process_data(int avg);
SYSTEM_SETTING_COEF cal_coef;
unsigned short data_array[4000];
double data_avg[4000];
int average;
void print_data();


int main(int argc, char* argv[])
{
	int c;
	int devnum;
	int int_time;
	int devices=0;
	bool dump_config_one=false;
	bool dump_config_all=false;
	bool first_device=false;
	bool get_int=false;
	bool set_int=false;
	bool measure_one=false;
	average=1;
	
	verbose=false;

	USB_SPEC usb_spec;
	devnum=-1;
	if(argc<2)
	{
		printusage();
		return(0);
	}
	while ((c = getopt(argc, argv, ("mcCvhlaid:I:n:"))) != EOF)
	{			
		switch (c)
		{
			case ('v'):
				verbose=true;
				break;
			case ('h'):
				printusage();
				break;
			case ('l'):
				devices=XSPEC_scan_usb_for_spec();
				printf("%d devices found\n", devices);
				XSPEC_print_Spec();
				break;
			case ('a'):
				first_device=true;
				break;	
			case ('d'):
				devnum=atoi(optarg);
				break;	
			case ('c'):
				dump_config_one=true;
				break;	
			case ('C'):
				dump_config_all=true;
				break;	
			case ('i'):
				get_int=true;
				break;	
			case ('I'):
				int_time=atoi(optarg);
				set_int=true;
				break;	
			case ('n'):
				average=atoi(optarg);
				break;	
			case ('m'):
				measure_one=true;
				break;	
			default:		// legal option - it was specified in optstring,
							// but it had no "case" handler 

				// Note: you may choose not to make this an error

				printf("WARNING:  no handler for option %c\n",c);
				return (-1);
				break;
		}
	}
	XSPEC_scan_usb_for_spec();
	if(dump_config_one)
	{
		if(devnum<0 && first_device==false)
		{
			printf("devnum %d not defined, use -d# or -a to define a device\n", devnum);
			goto EXIT;
		}
		if(first_device)
		{
			PRINT("Using first device\n");
			int ret=XSPEC_get_Spec(0, &usb_spec);
			devnum=usb_spec.devnum;
			PRINT("devnum=%d\n", devnum);
		}
		usb_spec.usb_dev=XSPEC_open_Spec(devnum);
		if(usb_spec.usb_dev==(usb_dev_handle *)-1)
		{
			printf("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		XSPEC_print_config(&usb_spec);
		XSPEC_close_Spec(&usb_spec);
		dump_config_one=false;
	}
	if(dump_config_all)
	{
		int total;
		total=XSPEC_getNumConnectedSpectrometers();
		for(int i=0;i<total;i++)
		{
			XSPEC_get_Spec(i, &usb_spec);
			devnum=usb_spec.devnum;
			usb_spec.usb_dev=XSPEC_open_Spec(devnum);
			if(usb_spec.usb_dev==(usb_dev_handle *)-1)
			{
				printf("Cannot open devnum %d\n", devnum);
				goto EXIT;
			}
			XSPEC_print_config(&usb_spec);
			XSPEC_close_Spec(&usb_spec);
		}
		dump_config_all=false;
	}
	if(get_int)
	{
		if(devnum<0 && first_device==false)
		{
			printf("devnum %d not defined, use -d# or -a to define a device\n", devnum);
			goto EXIT;
		}
		if(first_device)
		{
			PRINT("Using first device\n");
			int ret=XSPEC_get_Spec(0, &usb_spec);
			devnum=usb_spec.devnum;
			PRINT("devnum=%d\n", devnum);
		}
		usb_spec.usb_dev=XSPEC_open_Spec(devnum);
		if(usb_spec.usb_dev==(usb_dev_handle *)-1)
		{
			printf("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_getIntegrationTime(&usb_spec);
		printf("Integration Time for device Number %d=%d\n", devnum, temp_int_time);
		XSPEC_close_Spec(&usb_spec);
		get_int=false;
	}
	if(set_int)
	{
		if(devnum<0 && first_device==false)
		{
			printf("devnum %d not defined, use -d# or -a to define a device\n", devnum);
			goto EXIT;
		}
		if(first_device)
		{
			PRINT("Using first device\n");
			int ret=XSPEC_get_Spec(0, &usb_spec);
			devnum=usb_spec.devnum;
			PRINT("devnum=%d\n", devnum);
		}
		PRINT("set devnum %d , int_time=%d\n", devnum, int_time);
		usb_spec.usb_dev=XSPEC_open_Spec(devnum);
		if(usb_spec.usb_dev==(usb_dev_handle *)-1)
		{
			printf("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		XSPEC_setIntegrationTime(&usb_spec, int_time);
		XSPEC_close_Spec(&usb_spec);
		set_int=false;
	}
	if(measure_one)
	{

		if(devnum<0 && first_device==false)
		{
			printf("devnum %d not defined, use -d# or -a to define a device\n", devnum);
			goto EXIT;
		}
		if(first_device)
		{
			PRINT("Using first device\n");
			int ret=XSPEC_get_Spec(0, &usb_spec);
			devnum=usb_spec.devnum;
			PRINT("devnum=%d\n", devnum);
		}
		PRINT("set devnum %d \n", devnum);
		usb_spec.usb_dev=XSPEC_open_Spec(devnum);
		if(usb_spec.usb_dev==(usb_dev_handle *)-1)
		{
			printf("Cannot open devnum %d\n", devnum);
			goto EXIT;
		}
		int temp_int_time=XSPEC_getIntegrationTime(&usb_spec);
		printf("got int time %d\n", temp_int_time);
		XSPEC_get_SpecConfig0123(&usb_spec, (char *)&cal_coef, 2);
		printf("got XSPEC_get_SpecConfig0123 \n", temp_int_time);
		for(int j=0;j<average;j++)
		{
			XSPEC_measure(&usb_spec, data_array, 0x1234, temp_int_time);
			process_data(j);
			usleep(10000);
		}
		XSPEC_close_Spec(&usb_spec);
		print_data();

		measure_one=false;
	}

int XSPEC_measure(USB_SPEC *ptr,unsigned short *data_array, unsigned int id, unsigned int integ_time);

EXIT:
    XSPEC_usb_cleanup();
	return 0;
}

void printusage()
{
	printf("XSPEC_CONTROL usage:\n");
	printf("-v: verbose\n");
	printf("-l: list all USB XSPEC devices.\n");
	printf("-d #: use device number #, get the device number from -a.\n");
	printf("-a: use first xspec device on the list or if only if one attached. Precedance over -d #\n");
	printf("-i: get integration time\n");
	printf("-I #: Set Integration Time\n");
	printf("-c: Dump Config data\n");
	printf("-C: Dump all Config Data\n");
	printf("-m: Get a measurement. \n");
	printf("-n #: read # time and take average.\n");
}

void PRINT(const char *format, ...)
{
	if(verbose==false)
		return;
	va_list ap;
	va_start (ap, format);
	vprintf(format, ap);
	va_end (ap);
}
void print_data()
{
	double dark_value=0;
	double wave[4000];
	int i;
	for(i=cal_coef.dark_start;i<cal_coef.dark_end;i++)
		dark_value+=data_avg[i];
	dark_value=dark_value/(cal_coef.dark_end-cal_coef.dark_start);
	printf("dark value %lf\n", dark_value);
	PRINT("cal coef %lE %lE %lE %lE\n", cal_coef.user_coef[0], cal_coef.user_coef[1],cal_coef.user_coef[2],cal_coef.user_coef[3]);
	for(i=0;i<4000;i++)
	{
		wave[i]=cal_coef.user_coef[0]+cal_coef.user_coef[1]*i;
		wave[i]+=cal_coef.user_coef[2]*i*i+cal_coef.user_coef[3]*i*i*i;
	}
	printf("pixel   wavelength  value\n"); 
	for(i=cal_coef.data_start;i<cal_coef.data_end;i++)
	{
		printf("%d %lf %lf\n", i, wave[i], data_avg[i]-dark_value);
	}
}
void process_data(int avg)
{
	int i;
	double sum=0;
	for(i=0;i<4000;i++)
	{
		double temp;
		temp=data_array[i];
		sum+=temp;
		data_avg[i]=(data_avg[i]*avg+temp)/(avg+1);
	}
	sum=sum/4000;
	printf("average %lf\n", sum);
}



